package de.intevation.flys.artifacts.model.fixings;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.FixationArtifactAccess;

import de.intevation.flys.artifacts.math.fitting.Function;
import de.intevation.flys.artifacts.math.fitting.FunctionFactory;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.Parameters;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;

import org.apache.log4j.Logger;


/**
 * Facet to show the W|Q values.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixDerivateFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixDerivateFacet.class);

    private double currentKm;
    private double maxQ;

    /** Trivial Constructor. */
    public FixDerivateFacet() {
    }


    /**
     * @param name
     */
    public FixDerivateFacet(String name, String description) {
        super(0, name, description, ComputeType.ADVANCE, null, null);
    }


    public Object getData(Artifact artifact, CallContext context, double km) {
        this.currentKm = km;
        return getData(artifact, context);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixDerivateFacet.getData");
        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixationArtifactAccess access = new FixationArtifactAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixResult result = (FixResult) res.getData();

            String function = access.getFunction();
            Function ff = FunctionFactory.getInstance().getFunction(function);
            Function.Derivative fd = ff.getDerivative();

            Parameters params = result.getParameters();
            
            // Determine maxQ
            double[] maxQs = params
                .interpolate("km", this.currentKm, new String [] { "max_q" });
            if(maxQs != null) {
                maxQ = maxQs[0];
            }
            
            int row = params.binarySearch("km", currentKm, Math.pow(10, -4));
            if(row < 0) {
                row = -row - 1;
                logger.debug("getData: no direct hit in params.binarySearch");
            }
            String[] paramNames = ff.getParameterNames();
            int[] paramInd = params.columnIndices(paramNames);
            double[] coeffs = new double[paramNames.length];
            params.get(row, paramInd, coeffs);

            de.intevation.flys.artifacts.math.Function mf =
                fd.instantiate(coeffs);

            double maxQ = getMaxQ(result, currentKm);

            FixFunction fix = new FixFunction(
                "",
                ff.getDescription(),
                mf,
                maxQ);

            return fix;
        }
        else {
            logger.debug("Not an instance of FixationArtifact.");
            return null;
        }
    }


    protected double getMaxQ(FixResult result, double km) {
        return maxQ;
    }

    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixDerivateFacet deepCopy() {
        FixDerivateFacet copy = new FixDerivateFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
